@@ -12,16 +12,18 @@ use {
         },
     },
     crossbeam_channel::{Receiver, RecvTimeoutError, SendError, Sender},
+    rayon::prelude::*,
     serde::Serialize,
-    solana_account_decoder::{parse_token::spl_token_id_v2_0, UiAccount, UiAccountEncoding},
+    solana_account_decoder::{parse_token::spl_token_id, UiAccount, UiAccountEncoding},
     solana_client::{
         rpc_filter::RpcFilterType,
         rpc_response::{
             ProcessedSignatureResult, ReceivedSignatureResult, Response, RpcKeyedAccount,
-            RpcLogsResponse, RpcResponseContext, RpcSignatureResult, SlotInfo, SlotUpdate,
+            RpcLogsResponse, RpcResponseContext, RpcSignatureResult, RpcVote, SlotInfo, SlotUpdate,
         },
     },
     solana_measure::measure::Measure,
+    solana_rayon_threadlimit::get_thread_count,
     solana_runtime::{
         bank::{Bank, TransactionLogInfo},
         bank_forks::BankForks,
@@ -29,7 +31,7 @@ use {
     },
     solana_sdk::{
         account::{AccountSharedData, ReadableAccount},
-        clock::{Slot, UnixTimestamp},
+        clock::Slot,
         pubkey::Pubkey,
         signature::Signature,
         timing::timestamp,
@@ -37,15 +39,16 @@ use {
     },
     solana_vote_program::vote_state::Vote,
     std::{
+        cell::RefCell,
         collections::{HashMap, VecDeque},
         io::Cursor,
         iter, str,
         sync::{
-            atomic::{AtomicBool, Ordering},
-            Arc, RwLock, Weak,
+            atomic::{AtomicBool, AtomicUsize, Ordering},
+            Arc, Mutex, RwLock, Weak,
         },
         thread::{Builder, JoinHandle},
-        time::Duration,
+        time::{Duration, Instant},
     },
     tokio::sync::broadcast,
 };
@@ -69,13 +72,19 @@ fn get_transaction_logs(
     }
     logs
 }
+#[derive(Debug)]
+pub struct TimestampedNotificationEntry {
+    pub entry: NotificationEntry,
+    pub queued_at: Instant,
+}
 
-// A more human-friendly version of Vote, with the bank state signature base58 encoded.
-#[derive(Serialize, Deserialize, Debug)]
-pub struct RpcVote {
-    pub slots: Vec<Slot>,
-    pub hash: String,
-    pub timestamp: Option<UnixTimestamp>,
+impl From<NotificationEntry> for TimestampedNotificationEntry {
+    fn from(entry: NotificationEntry) -> Self {
+        TimestampedNotificationEntry {
+            entry,
+            queued_at: Instant::now(),
+        }
+    }
 }
 
 pub enum NotificationEntry {
@@ -130,7 +139,7 @@ fn check_commitment_and_notify<P, S, B, F, X>(
     commitment_slots: &CommitmentSlots,
     bank_method: B,
     filter_results: F,
-    notifier: &mut RpcNotifier,
+    notifier: &RpcNotifier,
     is_final: bool,
 ) -> bool
 where
@@ -180,6 +189,7 @@ pub struct RpcNotification {
     pub subscription_id: SubscriptionId,
     pub is_final: bool,
     pub json: Weak<String>,
+    pub created_at: Instant,
 }
 
 struct RecentItems {
@@ -224,8 +234,11 @@ impl RecentItems {
 
 struct RpcNotifier {
     sender: broadcast::Sender<RpcNotification>,
-    buf: Vec<u8>,
-    recent_items: RecentItems,
+    recent_items: Mutex<RecentItems>,
+}
+
+thread_local! {
+    static RPC_NOTIFIER_BUF: RefCell<Vec<u8>> = RefCell::new(Vec::new());
 }
 
 #[derive(Debug, Serialize)]
@@ -242,28 +255,32 @@ struct Notification<T> {
 }
 
 impl RpcNotifier {
-    fn notify<T>(&mut self, value: T, subscription: &SubscriptionInfo, is_final: bool)
+    fn notify<T>(&self, value: T, subscription: &SubscriptionInfo, is_final: bool)
     where
         T: serde::Serialize,
     {
-        self.buf.clear();
-        let notification = Notification {
-            jsonrpc: Some(jsonrpc_core::Version::V2),
-            method: subscription.method(),
-            params: NotificationParams {
-                result: value,
-                subscription: subscription.id(),
-            },
-        };
-        serde_json::to_writer(Cursor::new(&mut self.buf), &notification)
-            .expect("serialization never fails");
-        let buf_str = str::from_utf8(&self.buf).expect("json is always utf-8");
-        let buf_arc = Arc::new(String::from(buf_str));
+        let buf_arc = RPC_NOTIFIER_BUF.with(|buf| {
+            let mut buf = buf.borrow_mut();
+            buf.clear();
+            let notification = Notification {
+                jsonrpc: Some(jsonrpc_core::Version::V2),
+                method: subscription.method(),
+                params: NotificationParams {
+                    result: value,
+                    subscription: subscription.id(),
+                },
+            };
+            serde_json::to_writer(Cursor::new(&mut *buf), &notification)
+                .expect("serialization never fails");
+            let buf_str = str::from_utf8(&buf).expect("json is always utf-8");
+            Arc::new(String::from(buf_str))
+        });
 
         let notification = RpcNotification {
             subscription_id: subscription.id(),
             json: Arc::downgrade(&buf_arc),
             is_final,
+            created_at: Instant::now(),
         };
         // There is an unlikely case where this can fail: if the last subscription is closed
         // just as the notifier generates a notification for it.
@@ -272,7 +289,7 @@ impl RpcNotifier {
         inc_new_counter_info!("rpc-pubsub-messages", 1);
         inc_new_counter_info!("rpc-pubsub-bytes", buf_arc.len());
 
-        self.recent_items.push(buf_arc);
+        self.recent_items.lock().unwrap().push(buf_arc);
     }
 }
 
@@ -289,9 +306,7 @@ fn filter_account_result(
     // If last_modified_slot < last_notified_slot this means that we last notified for a fork
     // and should notify that the account state has been reverted.
     let results: Box<dyn Iterator<Item = UiAccount>> = if last_modified_slot != last_notified_slot {
-        if account.owner() == &spl_token_id_v2_0()
-            && params.encoding == UiAccountEncoding::JsonParsed
-        {
+        if account.owner() == &spl_token_id() && params.encoding == UiAccountEncoding::JsonParsed {
             Box::new(iter::once(get_parsed_token_account(
                 bank,
                 &params.pubkey,
@@ -342,8 +357,7 @@ fn filter_program_results(
             RpcFilterType::Memcmp(compare) => compare.bytes_match(account.data()),
         })
     });
-    let accounts: Box<dyn Iterator<Item = RpcKeyedAccount>> = if params.pubkey
-        == spl_token_id_v2_0()
+    let accounts: Box<dyn Iterator<Item = RpcKeyedAccount>> = if params.pubkey == spl_token_id()
         && params.encoding == UiAccountEncoding::JsonParsed
         && !accounts_is_empty
     {
@@ -419,9 +433,42 @@ fn initial_last_notified_slot(
     }
 }
 
-pub struct RpcSubscriptions {
-    notification_sender: Sender<NotificationEntry>,
+#[derive(Default)]
+struct PubsubNotificationStats {
+    since: Option<Instant>,
+    notification_entry_processing_count: u64,
+    notification_entry_processing_time_us: u64,
+}
 
+impl PubsubNotificationStats {
+    fn maybe_submit(&mut self) {
+        const SUBMIT_CADENCE: Duration = Duration::from_secs(2);
+        let elapsed = self.since.as_ref().map(Instant::elapsed);
+        if elapsed.map(|e| e < SUBMIT_CADENCE).unwrap_or_default() {
+            return;
+        }
+        datapoint_info!(
+            "pubsub_notification_entries",
+            (
+                "notification_entry_processing_count",
+                self.notification_entry_processing_count,
+                i64
+            ),
+            (
+                "notification_entry_processing_time_us",
+                self.notification_entry_processing_time_us,
+                i64
+            ),
+        );
+        *self = Self {
+            since: Some(Instant::now()),
+            ..Self::default()
+        };
+    }
+}
+
+pub struct RpcSubscriptions {
+    notification_sender: Sender<TimestampedNotificationEntry>,
     t_cleanup: Option<JoinHandle<()>>,
 
     exit: Arc<AtomicBool>,
@@ -483,29 +530,36 @@ impl RpcSubscriptions {
 
         let notifier = RpcNotifier {
             sender: broadcast_sender.clone(),
-            buf: Vec::new(),
-            recent_items: RecentItems::new(
+            recent_items: Mutex::new(RecentItems::new(
                 config.queue_capacity_items,
                 config.queue_capacity_bytes,
-            ),
+            )),
         };
+        let notification_threads = config.notification_threads;
         let t_cleanup = Builder::new()
             .name("solana-rpc-notifications".to_string())
             .spawn(move || {
-                Self::process_notifications(
-                    exit_clone,
-                    notifier,
-                    notification_receiver,
-                    subscriptions,
-                    bank_forks,
-                    block_commitment_cache,
-                    optimistically_confirmed_bank,
-                );
+                let pool = rayon::ThreadPoolBuilder::new()
+                    .num_threads(notification_threads.unwrap_or_else(get_thread_count))
+                    .thread_name(|i| format!("sol-sub-notif-{}", i))
+                    .build()
+                    .unwrap();
+                pool.install(|| {
+                    Self::process_notifications(
+                        exit_clone,
+                        notifier,
+                        notification_receiver,
+                        subscriptions,
+                        bank_forks,
+                        block_commitment_cache,
+                        optimistically_confirmed_bank,
+                    )
+                });
             })
             .unwrap();
 
         let control = SubscriptionControl::new(
-            PubSubConfig::default().max_active_subscriptions,
+            config.max_active_subscriptions,
             notification_sender.clone(),
             broadcast_sender,
         );
@@ -580,7 +634,7 @@ impl RpcSubscriptions {
     }
 
     fn enqueue_notification(&self, notification_entry: NotificationEntry) {
-        match self.notification_sender.send(notification_entry) {
+        match self.notification_sender.send(notification_entry.into()) {
             Ok(()) => (),
             Err(SendError(notification)) => {
                 warn!(
@@ -593,20 +647,23 @@ impl RpcSubscriptions {
 
     fn process_notifications(
         exit: Arc<AtomicBool>,
-        mut notifier: RpcNotifier,
-        notification_receiver: Receiver<NotificationEntry>,
+        notifier: RpcNotifier,
+        notification_receiver: Receiver<TimestampedNotificationEntry>,
         mut subscriptions: SubscriptionsTracker,
         bank_forks: Arc<RwLock<BankForks>>,
         block_commitment_cache: Arc<RwLock<BlockCommitmentCache>>,
         optimistically_confirmed_bank: Arc<RwLock<OptimisticallyConfirmedBank>>,
     ) {
+        let mut stats = PubsubNotificationStats::default();
+
         loop {
             if exit.load(Ordering::Relaxed) {
                 break;
             }
             match notification_receiver.recv_timeout(Duration::from_millis(RECEIVE_DELAY_MILLIS)) {
                 Ok(notification_entry) => {
-                    match notification_entry {
+                    let TimestampedNotificationEntry { entry, queued_at } = notification_entry;
+                    match entry {
                         NotificationEntry::Subscribed(params, id) => {
                             subscriptions.subscribe(params.clone(), id, || {
                                 initial_last_notified_slot(
@@ -673,7 +730,7 @@ impl RpcSubscriptions {
                                 subscriptions.commitment_watchers(),
                                 &bank_forks,
                                 &commitment_slots,
-                                &mut notifier,
+                                &notifier,
                                 "bank",
                             )
                         }
@@ -687,7 +744,7 @@ impl RpcSubscriptions {
                                 subscriptions.gossip_watchers(),
                                 &bank_forks,
                                 &commitment_slots,
-                                &mut notifier,
+                                &notifier,
                                 "gossip",
                             )
                         }
@@ -719,6 +776,9 @@ impl RpcSubscriptions {
                             }
                         }
                     }
+                    stats.notification_entry_processing_time_us +=
+                        queued_at.elapsed().as_micros() as u64;
+                    stats.notification_entry_processing_count += 1;
                 }
                 Err(RecvTimeoutError::Timeout) => {
                     // not a problem - try reading again
@@ -728,6 +788,7 @@ impl RpcSubscriptions {
                     break;
                 }
             }
+            stats.maybe_submit();
         }
     }
 
@@ -735,23 +796,24 @@ impl RpcSubscriptions {
         subscriptions: &HashMap<SubscriptionId, Arc<SubscriptionInfo>>,
         bank_forks: &Arc<RwLock<BankForks>>,
         commitment_slots: &CommitmentSlots,
-        notifier: &mut RpcNotifier,
+        notifier: &RpcNotifier,
         source: &'static str,
     ) {
         let mut total_time = Measure::start("notify_accounts_logs_programs_signatures");
-        let mut num_accounts_found = 0;
-        let mut num_accounts_notified = 0;
+        let num_accounts_found = AtomicUsize::new(0);
+        let num_accounts_notified = AtomicUsize::new(0);
 
-        let mut num_logs_found = 0;
-        let mut num_logs_notified = 0;
+        let num_logs_found = AtomicUsize::new(0);
+        let num_logs_notified = AtomicUsize::new(0);
 
-        let mut num_signatures_found = 0;
-        let mut num_signatures_notified = 0;
+        let num_signatures_found = AtomicUsize::new(0);
+        let num_signatures_notified = AtomicUsize::new(0);
 
-        let mut num_programs_found = 0;
-        let mut num_programs_notified = 0;
+        let num_programs_found = AtomicUsize::new(0);
+        let num_programs_notified = AtomicUsize::new(0);
 
-        for subscription in subscriptions.values() {
+        let subscriptions = subscriptions.into_par_iter();
+        subscriptions.for_each(|(_id, subscription)| {
             match subscription.params() {
                 SubscriptionParams::Account(params) => {
                     let notified = check_commitment_and_notify(
@@ -765,10 +827,10 @@ impl RpcSubscriptions {
                         false,
                     );
 
-                    num_accounts_found += 1;
+                    num_accounts_found.fetch_add(1, Ordering::Relaxed);
 
                     if notified {
-                        num_accounts_notified += 1;
+                        num_accounts_notified.fetch_add(1, Ordering::Relaxed);
                     }
                 }
                 SubscriptionParams::Logs(params) => {
@@ -782,10 +844,10 @@ impl RpcSubscriptions {
                         notifier,
                         false,
                     );
-                    num_logs_found += 1;
+                    num_logs_found.fetch_add(1, Ordering::Relaxed);
 
                     if notified {
-                        num_logs_notified += 1;
+                        num_logs_notified.fetch_add(1, Ordering::Relaxed);
                     }
                 }
                 SubscriptionParams::Program(params) => {
@@ -801,10 +863,10 @@ impl RpcSubscriptions {
                         notifier,
                         false,
                     );
-                    num_programs_found += 1;
+                    num_programs_found.fetch_add(1, Ordering::Relaxed);
 
                     if notified {
-                        num_programs_notified += 1;
+                        num_programs_notified.fetch_add(1, Ordering::Relaxed);
                     }
                 }
                 SubscriptionParams::Signature(params) => {
@@ -820,65 +882,97 @@ impl RpcSubscriptions {
                         notifier,
                         true, // Unsubscribe.
                     );
-                    num_signatures_found += 1;
+                    num_signatures_found.fetch_add(1, Ordering::Relaxed);
 
                     if notified {
-                        num_signatures_notified += 1;
+                        num_signatures_notified.fetch_add(1, Ordering::Relaxed);
                     }
                 }
                 _ => error!("wrong subscription type in alps map"),
             }
-        }
+        });
 
         total_time.stop();
 
-        let total_notified = num_accounts_notified
-            + num_logs_notified
-            + num_programs_notified
-            + num_signatures_notified;
+        let total_notified = num_accounts_notified.load(Ordering::Relaxed)
+            + num_logs_notified.load(Ordering::Relaxed)
+            + num_programs_notified.load(Ordering::Relaxed)
+            + num_signatures_notified.load(Ordering::Relaxed);
         let total_ms = total_time.as_ms();
         if total_notified > 0 || total_ms > 10 {
             debug!(
                 "notified({}): accounts: {} / {} logs: {} / {} programs: {} / {} signatures: {} / {}",
                 source,
-                num_accounts_found,
-                num_accounts_notified,
-                num_logs_found,
-                num_logs_notified,
-                num_programs_found,
-                num_programs_notified,
-                num_signatures_found,
-                num_signatures_notified,
+                num_accounts_found.load(Ordering::Relaxed),
+                num_accounts_notified.load(Ordering::Relaxed),
+                num_logs_found.load(Ordering::Relaxed),
+                num_logs_notified.load(Ordering::Relaxed),
+                num_programs_found.load(Ordering::Relaxed),
+                num_programs_notified.load(Ordering::Relaxed),
+                num_signatures_found.load(Ordering::Relaxed),
+                num_signatures_notified.load(Ordering::Relaxed),
             );
             inc_new_counter_info!("rpc-subscription-notify-bank-or-gossip", total_notified);
             datapoint_info!(
                 "rpc_subscriptions",
                 ("source", source.to_string(), String),
-                ("num_account_subscriptions", num_accounts_found, i64),
-                ("num_account_pubkeys_notified", num_accounts_notified, i64),
-                ("num_logs_subscriptions", num_logs_found, i64),
-                ("num_logs_notified", num_logs_notified, i64),
-                ("num_program_subscriptions", num_programs_found, i64),
-                ("num_programs_notified", num_programs_notified, i64),
-                ("num_signature_subscriptions", num_signatures_found, i64),
-                ("num_signatures_notified", num_signatures_notified, i64),
+                (
+                    "num_account_subscriptions",
+                    num_accounts_found.load(Ordering::Relaxed),
+                    i64
+                ),
+                (
+                    "num_account_pubkeys_notified",
+                    num_accounts_notified.load(Ordering::Relaxed),
+                    i64
+                ),
+                (
+                    "num_logs_subscriptions",
+                    num_logs_found.load(Ordering::Relaxed),
+                    i64
+                ),
+                (
+                    "num_logs_notified",
+                    num_logs_notified.load(Ordering::Relaxed),
+                    i64
+                ),
+                (
+                    "num_program_subscriptions",
+                    num_programs_found.load(Ordering::Relaxed),
+                    i64
+                ),
+                (
+                    "num_programs_notified",
+                    num_programs_notified.load(Ordering::Relaxed),
+                    i64
+                ),
+                (
+                    "num_signature_subscriptions",
+                    num_signatures_found.load(Ordering::Relaxed),
+                    i64
+                ),
+                (
+                    "num_signatures_notified",
+                    num_signatures_notified.load(Ordering::Relaxed),
+                    i64
+                ),
                 ("notifications_time", total_time.as_us() as i64, i64),
             );
             inc_new_counter_info!(
                 "rpc-subscription-counter-num_accounts_notified",
-                num_accounts_notified
+                num_accounts_notified.load(Ordering::Relaxed)
             );
             inc_new_counter_info!(
                 "rpc-subscription-counter-num_logs_notified",
-                num_logs_notified
+                num_logs_notified.load(Ordering::Relaxed)
             );
             inc_new_counter_info!(
                 "rpc-subscription-counter-num_programs_notified",
-                num_programs_notified
+                num_programs_notified.load(Ordering::Relaxed)
             );
             inc_new_counter_info!(
                 "rpc-subscription-counter-num_signatures_notified",
-                num_signatures_notified
+                num_signatures_notified.load(Ordering::Relaxed)
             );
         }
     }
